﻿using System.Drawing;

using Steema.TeeChart;
using Steema.TeeChart.Styles;

namespace TestProject
{
    /// <summary>
    /// 커스텀 바
    /// </summary>
    public class CustomBar : Bar
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 크기 값 리스트
        /// </summary>
        private ValueList sizeValueList;

        /// <summary>
        /// 크기
        /// </summary>
        private int size;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 크기 값 리스트 - SizeValues

        /// <summary>
        /// 크기 값 리스트
        /// </summary>
        public ValueList SizeValues
        {
            get
            {
                return this.sizeValueList;
            }
            set
            {
                SetValueList(this.sizeValueList, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Consturctor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomBar()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomBar() : this(null)
        {
        }

        #endregion
        #region 생성자 - CustomBar(chart)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="chart">차트</param>
        public CustomBar(Chart chart) : base(chart) 
        {
            this.sizeValueList = new ValueList(this, "xSize");
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 그리기 - DrawValue(valueIndex)

        /// <summary>
        /// 값 그리기
        /// </summary>
        /// <param name="valueIndex">값 인덱스</param>
        public override void DrawValue(int valueIndex)
        {
            if(valueIndex == firstVisible)
            {
                this.size = IBarSize;
            }

            InternalCalcBarSize(valueIndex);

            base.DrawValue(valueIndex);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 클릭시 처리하기 (내부용) - InternalClicked(valueIndex, point)

        /// <summary>
        /// 클릭시 처리하기 (내부용)
        /// </summary>
        /// <param name="valueIndex">값 인덱스</param>
        /// <param name="point">포인트</param>
        /// <returns>처리 결과</returns>
        protected override bool InternalClicked(int valueIndex, Point point)
        {
            InternalCalcBarSize(valueIndex);

            return base.InternalClicked(valueIndex, point);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 바 크기 계산하기 (내부용) - InternalCalcBarSize(valueIndex)

        /// <summary>
        /// 바 크기 계산하기 (내부용)
        /// </summary>
        /// <param name="valueIndex">값 인덱스</param>
        private void InternalCalcBarSize(int valueIndex)
        {
            if(this.sizeValueList[valueIndex] == 0)
            {
                IBarSize = this.size;
            }
            else
            {
                IBarSize = GetHorizAxis.CalcSizeValue(this.sizeValueList[valueIndex]);
            }
        }

        #endregion
    }
}