﻿using System;
using System.Drawing;
using System.Windows.Forms;

using Steema.TeeChart.Drawing;
using Steema.TeeChart.Styles;
using Steema.TeeChart.Tools;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 커스텀 바
        /// </summary>
        private CustomBar customBar;

        /// <summary>
        /// 주석
        /// </summary>
        private Annotation annotation;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            Text = "Bar 클래스 : 바 위치/크기 설정하기";

            this.tChart.Panel.Pen = new ChartPen(Color.Black);

            #region 커스텀 바를 설정한다.

            this.customBar = new CustomBar();

            this.customBar.FillSampleValues(6);

            this.customBar.SizeValues[0] = 0.4;
            this.customBar.SizeValues[2] = 0.1;
            this.customBar.SizeValues[4] = 1.1;

            this.customBar.XValues[3] = 2.8;

            this.customBar.Colors[1] = Color.Blue;
            this.customBar.Colors[4] = Color.Yellow;

            this.customBar.Marks.Visible = false;
    
            this.customBar.Cursor = Cursors.Hand;

            this.tChart.Series.Add(this.customBar);

            #endregion
            #region 주석을 추가한다.

            this.annotation = new Annotation();

            this.annotation.Shape.CustomPosition = true;

            this.tChart.Tools.Add(this.annotation);

            #endregion
            #region 바 숫자 UP/DOWN을 설정한다.

            this.barNumericUpDown.Minimum = 0;
            this.barNumericUpDown.Maximum = this.customBar.Count - 1;
            this.barNumericUpDown.Value   = 0;

            #endregion
            #region 위치 트랙바를 설정한다.

            this.positionTrackBar.Maximum       =  this.customBar.Count * 100;
            this.positionTrackBar.TickFrequency =  this.positionTrackBar.Maximum / 10;
            this.positionTrackBar.Minimum       = -this.positionTrackBar.Maximum;

            #endregion
            #region 크기 트랙바를 설정한다.

            this.sizeTrackBar.Minimum       = 0;
            this.sizeTrackBar.Maximum       = 300;
            this.sizeTrackBar.Value         = 100;
            this.sizeTrackBar.TickFrequency = 20;

            #endregion
            #region 이벤트를 설정한다.

            Load                               += Form_Load;
            this.barNumericUpDown.ValueChanged += barNumericUpDown_ValueChanged;
            this.positionTrackBar.ValueChanged += positionTrackBar_Scroll;
            this.sizeTrackBar.ValueChanged     += sizeTrackBar_Scroll;
            this.tChart.ClickSeries            += tChart_ClickSeries;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            barNumericUpDown_ValueChanged(null, null);
        }

        #endregion
        #region 바 숫자 UP/DOWN 값 변경시 처리하기 - barNumericUpDown_ValueChanged(sender, e)

        /// <summary>
        /// 바 숫자 UP/DOWN 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void barNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            int index = Convert.ToInt32(this.barNumericUpDown.Value);

            this.positionTrackBar.Value = Convert.ToInt32(this.customBar.XValues   [index] * 100);
            this.sizeTrackBar.Value     = Convert.ToInt32(this.customBar.SizeValues[index] * 100);

            RefreshAnnotation();
        }

        #endregion
        #region 위치 트랙바 스크롤시 처리하기 - positionTrackBar_Scroll(sender, e)

        /// <summary>
        /// 위치 트랙바 스크롤시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void positionTrackBar_Scroll(object sender, EventArgs e)
        {
            int index = Convert.ToInt32(this.barNumericUpDown.Value);

            this.customBar.XValues[index] = this.positionTrackBar.Value / 100.0;

            this.customBar.Repaint();

            this.positionValueLabel.Text = this.customBar.XValues[index].ToString();

            RefreshAnnotation();
        }

        #endregion
        #region 크기 트랙바 스크롤시 처리하기 - sizeTrackBar_Scroll(sender, e)

        /// <summary>
        /// 크기 트랙바 스크롤시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sizeTrackBar_Scroll(object sender, EventArgs e)
        {
            int index = Convert.ToInt32(this.barNumericUpDown.Value);

            this.customBar.SizeValues[index] = this.sizeTrackBar.Value / 100.0;

            this.customBar.Repaint();

            this.sizeValueLabel.Text = this.customBar.SizeValues[index].ToString();
        }

        #endregion
        #region TChart 시리즈 클릭시 처리하기 - tChart_ClickSeries(sender, series, valueIndex, e)

        /// <summary>
        /// TChart 시리즈 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="series">시리즈</param>
        /// <param name="valueIndex">값 인덱스</param>
        /// <param name="e">이벤트 인자</param>
        private void tChart_ClickSeries(object sender, Series series, int valueIndex, MouseEventArgs e)
        {
            if(valueIndex >= 0)
            {
                this.barNumericUpDown.Value = valueIndex;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 주석 갱신하기 - RefreshAnnotation()

        /// <summary>
        /// 주석 갱신하기
        /// </summary>
        private void RefreshAnnotation()
        {
            int index = Convert.ToInt32(this.barNumericUpDown.Value);

            this.annotation.Text       = index.ToString();
            this.annotation.Shape.Top  = this.tChart.Height - 28;
            this.annotation.Shape.Left = this.customBar.CalcXPosValue(this.customBar.XValues[index]) - 8;
        }

        #endregion
    }
}